/*
 * Copyright 2009 SpringSource
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.springsource.bundlor.support.classpath;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertNull;
import static org.junit.Assert.assertTrue;

import java.io.File;
import java.io.IOException;
import java.util.Iterator;

import org.junit.Test;

import com.springsource.bundlor.ClassPathEntry;

public class FileSystemClassPathTests {

    private final FileSystemClassPath classPath;

    public FileSystemClassPathTests() throws IOException {
        classPath = new FileSystemClassPath(new File("src/test/resources"));
    }

    @Test
    public void entry() {
        assertTrue(this.classPath.getEntry("input.jar") instanceof FileSystemClassPathEntry);
        assertNull(this.classPath.getEntry("test.properties"));
    }

    @Test
    public void iterator() {
        Iterator<ClassPathEntry> i = this.classPath.iterator();
        assertTrue(i.hasNext());
        ClassPathEntry entry = i.next();
        assertNotNull(entry);
        assertTrue(entry instanceof FileSystemClassPathEntry);
    }

    @Test
    public void classPathEntry() {
        ClassPathEntry entry = this.classPath.getEntry("input.jar");
        assertNotNull(entry.getInputStream());
        assertNotNull(entry.getReader());
        assertFalse(entry.isDirectory());
        assertEquals("input.jar", entry.getName());
        assertTrue(this.classPath.getEntry("/").isDirectory());
    }
}
